@@ -20,18 +20,18 @@ Tests for Volume Code.
 
 """
 
-import os
 import datetime
+import os
 
 import mox
 import shutil
 import tempfile
 
 from cinder import context
-from cinder import exception
 from cinder import db
+from cinder import exception
 from cinder import flags
-from cinder.tests.image import fake as fake_image
+from cinder.image import image_utils
 from cinder.openstack.common import importutils
 from cinder.openstack.common.notifier import api as notifier_api
 from cinder.openstack.common.notifier import test_notifier
@@ -39,6 +39,10 @@ from cinder.openstack.common import rpc
 import cinder.policy
 from cinder import quota
 from cinder import test
+from cinder.tests import fake_flags
+from cinder.tests.image import fake as fake_image
+from cinder.volume import configuration as conf
+from cinder.volume import driver
 from cinder.volume import iscsi
 
 QUOTAS = quota.QUOTAS
@@ -84,6 +88,7 @@ class VolumeTestCase(test.TestCase):
         vol['availability_zone'] = FLAGS.storage_availability_zone
         vol['status'] = "creating"
         vol['attach_status'] = "detached"
+        vol['host'] = FLAGS.host
         if metadata is not None:
             vol['metadata'] = metadata
         return db.volume_create(context.get_admin_context(), vol)
@@ -91,13 +96,13 @@ class VolumeTestCase(test.TestCase):
     def test_create_delete_volume(self):
         """Test volume can be created and deleted."""
         # Need to stub out reserve, commit, and rollback
-        def fake_reserve(context, expire=None, **deltas):
+        def fake_reserve(context, expire=None, project_id=None, **deltas):
             return ["RESERVATION"]
 
-        def fake_commit(context, reservations):
+        def fake_commit(context, reservations, project_id=None):
             pass
 
-        def fake_rollback(context, reservations):
+        def fake_rollback(context, reservations, project_id=None):
             pass
 
         self.stubs.Set(QUOTAS, "reserve", fake_reserve)
@@ -113,6 +118,9 @@ class VolumeTestCase(test.TestCase):
                          volume_id).id)
 
         self.volume.delete_volume(self.context, volume_id)
+        vol = db.volume_get(context.get_admin_context(read_deleted='yes'),
+                            volume_id)
+        self.assertEquals(vol['status'], 'deleted')
         self.assertEquals(len(test_notifier.NOTIFICATIONS), 4)
         self.assertRaises(exception.NotFound,
                           db.volume_get,
@@ -135,6 +143,75 @@ class VolumeTestCase(test.TestCase):
                           self.context,
                           volume_id)
 
+    def test_create_volume_with_invalid_metadata(self):
+        """Test volume create with too much metadata fails."""
+        volume_api = cinder.volume.api.API()
+        test_meta = {'fake_key': 'fake_value' * 256}
+        self.assertRaises(exception.InvalidVolumeMetadataSize,
+                          volume_api.create,
+                          self.context,
+                          1,
+                          'name',
+                          'description',
+                          None,
+                          None,
+                          None,
+                          test_meta)
+
+    def test_create_volume_with_volume_type(self):
+        """Test volume creation with default volume type."""
+        def fake_reserve(context, expire=None, project_id=None, **deltas):
+            return ["RESERVATION"]
+
+        def fake_commit(context, reservations, project_id=None):
+            pass
+
+        def fake_rollback(context, reservations, project_id=None):
+            pass
+
+        self.stubs.Set(QUOTAS, "reserve", fake_reserve)
+        self.stubs.Set(QUOTAS, "commit", fake_commit)
+        self.stubs.Set(QUOTAS, "rollback", fake_rollback)
+
+        volume_api = cinder.volume.api.API()
+
+        # Create volume with default volume type while default
+        # volume type doesn't exist, volume_type_id should be NULL
+        volume = volume_api.create(self.context,
+                                   1,
+                                   'name',
+                                   'description')
+        self.assertEquals(volume['volume_type_id'], None)
+
+        # Create default volume type
+        vol_type = fake_flags.def_vol_type
+        db.volume_type_create(context.get_admin_context(),
+                              dict(name=vol_type, extra_specs={}))
+
+        db_vol_type = db.volume_type_get_by_name(context.get_admin_context(),
+                                                 vol_type)
+
+        # Create volume with default volume type
+        volume = volume_api.create(self.context,
+                                   1,
+                                   'name',
+                                   'description')
+        self.assertEquals(volume['volume_type_id'], db_vol_type.get('id'))
+
+        # Create volume with specific volume type
+        vol_type = 'test'
+        db.volume_type_create(context.get_admin_context(),
+                              dict(name=vol_type, extra_specs={}))
+        db_vol_type = db.volume_type_get_by_name(context.get_admin_context(),
+                                                 vol_type)
+
+        volume = volume_api.create(self.context,
+                                   1,
+                                   'name',
+                                   'description',
+                                   volume_type=db_vol_type)
+        self.assertEquals(volume['volume_type_id'], db_vol_type.get('id'))
+
     def test_delete_busy_volume(self):
         """Test volume survives deletion if driver reports it as busy."""
         volume = self._create_volume()
@@ -142,8 +219,9 @@ class VolumeTestCase(test.TestCase):
         self.volume.create_volume(self.context, volume_id)
 
         self.mox.StubOutWithMock(self.volume.driver, 'delete_volume')
-        self.volume.driver.delete_volume(mox.IgnoreArg()) \
-                                              .AndRaise(exception.VolumeIsBusy)
+        self.volume.driver.delete_volume(
+            mox.IgnoreArg()).AndRaise(exception.VolumeIsBusy(
+                                      volume_name='fake'))
         self.mox.ReplayAll()
         res = self.volume.delete_volume(self.context, volume_id)
         self.assertEqual(True, res)
@@ -158,7 +236,7 @@ class VolumeTestCase(test.TestCase):
         """Test volume can be created from a snapshot."""
         volume_src = self._create_volume()
         self.volume.create_volume(self.context, volume_src['id'])
-        snapshot_id = self._create_snapshot(volume_src['id'])
+        snapshot_id = self._create_snapshot(volume_src['id'])['id']
         self.volume.create_snapshot(self.context, volume_src['id'],
                                     snapshot_id)
         volume_dst = self._create_volume(0, snapshot_id)
@@ -167,9 +245,9 @@ class VolumeTestCase(test.TestCase):
                          db.volume_get(
                              context.get_admin_context(),
                              volume_dst['id']).id)
-        self.assertEqual(snapshot_id, db.volume_get(
-                context.get_admin_context(),
-                volume_dst['id']).snapshot_id)
+        self.assertEqual(snapshot_id,
+                         db.volume_get(context.get_admin_context(),
+                                       volume_dst['id']).snapshot_id)
 
         self.volume.delete_volume(self.context, volume_dst['id'])
         self.volume.delete_snapshot(self.context, snapshot_id)
@@ -215,6 +293,23 @@ class VolumeTestCase(test.TestCase):
                           self.context,
                           volume_id)
 
+    @test.skip_test
+    def test_preattach_status_volume(self):
+        """Ensure volume goes into pre-attaching state"""
+        instance_uuid = '12345678-1234-5678-1234-567812345678'
+        mountpoint = "/dev/sdf"
+        volume = db.volume_create(self.context, {'size': 1,
+                                                 'status': 'available'})
+        volume_id = volume['id']
+
+        volume_api = cinder.volume.api.API()
+        volume_api.attach(self.context, volume, instance_uuid, mountpoint)
+
+        vol = db.volume_get(self.context, volume_id)
+        self.assertEqual(vol['status'], "available")
+        self.assertEqual(vol['attach_status'], None)
+        self.assertEqual(vol['instance_uuid'], None)
+
     def test_concurrent_volumes_get_different_targets(self):
         """Ensure multiple concurrent volumes get different targets."""
         volume_ids = []
@@ -250,19 +345,22 @@ class VolumeTestCase(test.TestCase):
         snap['project_id'] = 'fake'
         snap['volume_id'] = volume_id
         snap['status'] = "creating"
-        return db.snapshot_create(context.get_admin_context(), snap)['id']
+        return db.snapshot_create(context.get_admin_context(), snap)
 
     def test_create_delete_snapshot(self):
         """Test snapshot can be created and deleted."""
         volume = self._create_volume()
         self.volume.create_volume(self.context, volume['id'])
-        snapshot_id = self._create_snapshot(volume['id'])
+        snapshot_id = self._create_snapshot(volume['id'])['id']
         self.volume.create_snapshot(self.context, volume['id'], snapshot_id)
         self.assertEqual(snapshot_id,
                          db.snapshot_get(context.get_admin_context(),
                                          snapshot_id).id)
 
         self.volume.delete_snapshot(self.context, snapshot_id)
+        snap = db.snapshot_get(context.get_admin_context(read_deleted='yes'),
+                               snapshot_id)
+        self.assertEquals(snap['status'], 'deleted')
         self.assertRaises(exception.NotFound,
                           db.snapshot_get,
                           self.context,
@@ -318,7 +416,7 @@ class VolumeTestCase(test.TestCase):
         """Test volume can't be deleted with dependent snapshots."""
         volume = self._create_volume()
         self.volume.create_volume(self.context, volume['id'])
-        snapshot_id = self._create_snapshot(volume['id'])
+        snapshot_id = self._create_snapshot(volume['id'])['id']
         self.volume.create_snapshot(self.context, volume['id'], snapshot_id)
         self.assertEqual(snapshot_id,
                          db.snapshot_get(context.get_admin_context(),
@@ -340,7 +438,7 @@ class VolumeTestCase(test.TestCase):
         """Test snapshot can be created and deleted."""
         volume = self._create_volume()
         self.volume.create_volume(self.context, volume['id'])
-        snapshot_id = self._create_snapshot(volume['id'])
+        snapshot_id = self._create_snapshot(volume['id'])['id']
         self.volume.create_snapshot(self.context, volume['id'], snapshot_id)
         snapshot = db.snapshot_get(context.get_admin_context(),
                                    snapshot_id)
@@ -388,12 +486,13 @@ class VolumeTestCase(test.TestCase):
         volume = self._create_volume()
         volume_id = volume['id']
         self.volume.create_volume(self.context, volume_id)
-        snapshot_id = self._create_snapshot(volume_id)
+        snapshot_id = self._create_snapshot(volume_id)['id']
         self.volume.create_snapshot(self.context, volume_id, snapshot_id)
 
         self.mox.StubOutWithMock(self.volume.driver, 'delete_snapshot')
-        self.volume.driver.delete_snapshot(mox.IgnoreArg()) \
-                                            .AndRaise(exception.SnapshotIsBusy)
+        self.volume.driver.delete_snapshot(
+            mox.IgnoreArg()).AndRaise(
+                exception.SnapshotIsBusy(snapshot_name='fake'))
         self.mox.ReplayAll()
         self.volume.delete_snapshot(self.context, snapshot_id)
         snapshot_ref = db.snapshot_get(self.context, snapshot_id)
@@ -411,12 +510,17 @@ class VolumeTestCase(test.TestCase):
         def fake_local_path(volume):
             return dst_path
 
-        def fake_copy_image_to_volume(context, volume, image_id):
+        def fake_copy_image_to_volume(context, volume,
+                                      image_service, image_id):
+            pass
+
+        def fake_fetch_to_raw(context, image_service, image_id, vol_path):
             pass
 
         dst_fd, dst_path = tempfile.mkstemp()
         os.close(dst_fd)
         self.stubs.Set(self.volume.driver, 'local_path', fake_local_path)
+        self.stubs.Set(image_utils, 'fetch_to_raw', fake_fetch_to_raw)
         if fakeout_copy_image_to_volume:
             self.stubs.Set(self.volume, '_copy_image_to_volume',
                            fake_copy_image_to_volume)
@@ -424,13 +528,14 @@ class VolumeTestCase(test.TestCase):
         image_id = 'c905cedb-7281-47e4-8a62-f26bc5fc4c77'
         volume_id = 1
         # creating volume testdata
-        db.volume_create(self.context, {'id': volume_id,
-                            'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                            'display_description': 'Test Desc',
-                            'size': 20,
-                            'status': 'creating',
-                            'instance_uuid': None,
-                            'host': 'dummy'})
+        db.volume_create(self.context,
+                         {'id': volume_id,
+                          'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                          'display_description': 'Test Desc',
+                          'size': 20,
+                          'status': 'creating',
+                          'instance_uuid': None,
+                          'host': 'dummy'})
         try:
             self.volume.create_volume(self.context,
                                       volume_id,
@@ -443,11 +548,6 @@ class VolumeTestCase(test.TestCase):
             db.volume_destroy(self.context, volume_id)
             os.unlink(dst_path)
 
-    def test_create_volume_from_image_status_downloading(self):
-        """Verify that before copying image to volume, it is in downloading
-        state."""
-        self._create_volume_from_image('downloading', True)
-
     def test_create_volume_from_image_status_available(self):
         """Verify that before copying image to volume, it is in available
         state."""
@@ -464,17 +564,18 @@ class VolumeTestCase(test.TestCase):
         image_id = 'aaaaaaaa-0000-0000-0000-000000000000'
         # creating volume testdata
         volume_id = 1
-        db.volume_create(self.context, {'id': volume_id,
-                             'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                             'display_description': 'Test Desc',
-                             'size': 20,
-                             'status': 'creating',
-                             'host': 'dummy'})
+        db.volume_create(self.context,
+                         {'id': volume_id,
+                          'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                          'display_description': 'Test Desc',
+                          'size': 20,
+                          'status': 'creating',
+                          'host': 'dummy'})
 
         self.assertRaises(exception.ImageNotFound,
                           self.volume.create_volume,
                           self.context,
-                          volume_id,
+                          volume_id, None, None, None,
                           None,
                           image_id)
         volume = db.volume_get(self.context, volume_id)
@@ -492,22 +593,27 @@ class VolumeTestCase(test.TestCase):
 
         self.stubs.Set(self.volume.driver, 'local_path', fake_local_path)
 
-        image_id = '70a599e0-31e7-49b7-b260-868f441e862b'
+        image_meta = {
+            'id': '70a599e0-31e7-49b7-b260-868f441e862b',
+            'container_format': 'bare',
+            'disk_format': 'raw'}
+
         # creating volume testdata
         volume_id = 1
-        db.volume_create(self.context, {'id': volume_id,
-                             'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                             'display_description': 'Test Desc',
-                             'size': 20,
-                             'status': 'uploading',
-                             'instance_uuid': None,
-                             'host': 'dummy'})
+        db.volume_create(self.context,
+                         {'id': volume_id,
+                          'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                          'display_description': 'Test Desc',
+                          'size': 20,
+                          'status': 'uploading',
+                          'instance_uuid': None,
+                          'host': 'dummy'})
 
         try:
             # start test
             self.volume.copy_volume_to_image(self.context,
-                                                volume_id,
-                                                image_id)
+                                             volume_id,
+                                             image_meta)
 
             volume = db.volume_get(self.context, volume_id)
             self.assertEqual(volume['status'], 'available')
@@ -525,25 +631,27 @@ class VolumeTestCase(test.TestCase):
 
         self.stubs.Set(self.volume.driver, 'local_path', fake_local_path)
 
-        #image_id = '70a599e0-31e7-49b7-b260-868f441e862b'
-        image_id = 'a440c04b-79fa-479c-bed1-0b816eaec379'
+        image_meta = {
+            'id': 'a440c04b-79fa-479c-bed1-0b816eaec379',
+            'container_format': 'bare',
+            'disk_format': 'raw'}
         # creating volume testdata
         volume_id = 1
-        db.volume_create(self.context,
-                         {'id': volume_id,
-                         'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                         'display_description': 'Test Desc',
-                         'size': 20,
-                         'status': 'uploading',
-                         'instance_uuid':
-                            'b21f957d-a72f-4b93-b5a5-45b1161abb02',
-                         'host': 'dummy'})
+        db.volume_create(
+            self.context,
+            {'id': volume_id,
+             'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+             'display_description': 'Test Desc',
+             'size': 20,
+             'status': 'uploading',
+             'instance_uuid': 'b21f957d-a72f-4b93-b5a5-45b1161abb02',
+             'host': 'dummy'})
 
         try:
             # start test
             self.volume.copy_volume_to_image(self.context,
-                                                volume_id,
-                                                image_id)
+                                             volume_id,
+                                             image_meta)
 
             volume = db.volume_get(self.context, volume_id)
             self.assertEqual(volume['status'], 'in-use')
@@ -561,15 +669,19 @@ class VolumeTestCase(test.TestCase):
 
         self.stubs.Set(self.volume.driver, 'local_path', fake_local_path)
 
-        image_id = 'aaaaaaaa-0000-0000-0000-000000000000'
+        image_meta = {
+            'id': 'aaaaaaaa-0000-0000-0000-000000000000',
+            'container_format': 'bare',
+            'disk_format': 'raw'}
         # creating volume testdata
         volume_id = 1
-        db.volume_create(self.context, {'id': volume_id,
-                             'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
-                             'display_description': 'Test Desc',
-                             'size': 20,
-                             'status': 'in-use',
-                             'host': 'dummy'})
+        db.volume_create(self.context,
+                         {'id': volume_id,
+                          'updated_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                          'display_description': 'Test Desc',
+                          'size': 20,
+                          'status': 'in-use',
+                          'host': 'dummy'})
 
         try:
             # start test
@@ -577,7 +689,7 @@ class VolumeTestCase(test.TestCase):
                               self.volume.copy_volume_to_image,
                               self.context,
                               volume_id,
-                              image_id)
+                              image_meta)
 
             volume = db.volume_get(self.context, volume_id)
             self.assertEqual(volume['status'], 'available')
@@ -594,14 +706,16 @@ class VolumeTestCase(test.TestCase):
                 pass
 
             def show(self, context, image_id):
-                return {'size': 2 * 1024 * 1024 * 1024}
+                return {'size': 2 * 1024 * 1024 * 1024,
+                        'disk_format': 'raw',
+                        'container_format': 'bare'}
 
         image_id = '70a599e0-31e7-49b7-b260-868f441e862b'
 
         try:
             volume_id = None
             volume_api = cinder.volume.api.API(
-                                            image_service=_FakeImageService())
+                image_service=_FakeImageService())
             volume = volume_api.create(self.context, 2, 'name', 'description',
                                        image_id=1)
             volume_id = volume['id']
@@ -618,7 +732,9 @@ class VolumeTestCase(test.TestCase):
                 pass
 
             def show(self, context, image_id):
-                return {'size': 2 * 1024 * 1024 * 1024 + 1}
+                return {'size': 2 * 1024 * 1024 * 1024 + 1,
+                        'disk_format': 'raw',
+                        'container_format': 'bare'}
 
         image_id = '70a599e0-31e7-49b7-b260-868f441e862b'
 
@@ -630,13 +746,13 @@ class VolumeTestCase(test.TestCase):
                           'name', 'description', image_id=1)
 
     def _do_test_create_volume_with_size(self, size):
-        def fake_reserve(context, expire=None, **deltas):
+        def fake_reserve(context, expire=None, project_id=None, **deltas):
             return ["RESERVATION"]
 
-        def fake_commit(context, reservations):
+        def fake_commit(context, reservations, project_id=None):
             pass
 
-        def fake_rollback(context, reservations):
+        def fake_rollback(context, reservations, project_id=None):
             pass
 
         self.stubs.Set(QUOTAS, "reserve", fake_reserve)
@@ -660,13 +776,13 @@ class VolumeTestCase(test.TestCase):
         self._do_test_create_volume_with_size('2')
 
     def test_create_volume_with_bad_size(self):
-        def fake_reserve(context, expire=None, **deltas):
+        def fake_reserve(context, expire=None, project_id=None, **deltas):
             return ["RESERVATION"]
 
-        def fake_commit(context, reservations):
+        def fake_commit(context, reservations, project_id=None):
             pass
 
-        def fake_rollback(context, reservations):
+        def fake_rollback(context, reservations, project_id=None):
             pass
 
         self.stubs.Set(QUOTAS, "reserve", fake_reserve)
@@ -717,6 +833,30 @@ class VolumeTestCase(test.TestCase):
         volume = db.volume_get(self.context, volume['id'])
         self.assertEqual(volume['status'], "in-use")
 
+    def test_volume_api_update(self):
+        # create a raw vol
+        volume = self._create_volume()
+        # use volume.api to update name
+        volume_api = cinder.volume.api.API()
+        update_dict = {'display_name': 'test update name'}
+        volume_api.update(self.context, volume, update_dict)
+        # read changes from db
+        vol = db.volume_get(context.get_admin_context(), volume['id'])
+        self.assertEquals(vol['display_name'], 'test update name')
+
+    def test_volume_api_update_snapshot(self):
+        # create raw snapshot
+        volume = self._create_volume()
+        snapshot = self._create_snapshot(volume['id'])
+        self.assertEquals(snapshot['display_name'], None)
+        # use volume.api to update name
+        volume_api = cinder.volume.api.API()
+        update_dict = {'display_name': 'test update name'}
+        volume_api.update_snapshot(self.context, snapshot, update_dict)
+        # read changes from db
+        snap = db.snapshot_get(context.get_admin_context(), snapshot['id'])
+        self.assertEquals(snap['display_name'], 'test update name')
+
 
 class DriverTestCase(test.TestCase):
     """Base Test class for Drivers."""
@@ -760,7 +900,7 @@ class DriverTestCase(test.TestCase):
 
 class VolumeDriverTestCase(DriverTestCase):
     """Test case for VolumeDriver"""
-    driver_name = "cinder.volume.driver.VolumeDriver"
+    driver_name = "cinder.volume.drivers.lvm.LVMVolumeDriver"
 
     def test_delete_busy_volume(self):
         """Test deleting a busy volume."""
@@ -782,7 +922,7 @@ class VolumeDriverTestCase(DriverTestCase):
 
 class ISCSITestCase(DriverTestCase):
     """Test Case for ISCSIDriver"""
-    driver_name = "cinder.volume.driver.ISCSIDriver"
+    driver_name = "cinder.volume.drivers.lvm.LVMISCSIDriver"
 
     def _attach_volume(self):
         """Attach volumes to an instance. """
@@ -803,9 +943,54 @@ class ISCSITestCase(DriverTestCase):
 
         return volume_id_list
 
-    def test_check_for_export_with_no_volume(self):
-        instance_uuid = '12345678-1234-5678-1234-567812345678'
-        self.volume.check_for_export(self.context, instance_uuid)
+    def test_do_iscsi_discovery(self):
+        configuration = mox.MockObject(conf.Configuration)
+        configuration.iscsi_ip_address = '0.0.0.0'
+        configuration.append_config_values(mox.IgnoreArg())
+
+        iscsi_driver = driver.ISCSIDriver(configuration=configuration)
+        iscsi_driver._execute = lambda *a, **kw: \
+            ("%s dummy" % FLAGS.iscsi_ip_address, '')
+        volume = {"name": "dummy",
+                  "host": "0.0.0.0"}
+        iscsi_driver._do_iscsi_discovery(volume)
+
+    def test_get_iscsi_properties(self):
+        volume = {"provider_location": '',
+                  "id": "0",
+                  "provider_auth": "a b c"}
+        iscsi_driver = driver.ISCSIDriver()
+        iscsi_driver._do_iscsi_discovery = lambda v: "0.0.0.0:0000,0 iqn:iqn 0"
+        result = iscsi_driver._get_iscsi_properties(volume)
+        self.assertEquals(result["target_portal"], "0.0.0.0:0000")
+        self.assertEquals(result["target_iqn"], "iqn:iqn")
+        self.assertEquals(result["target_lun"], 0)
+
+    def test_get_volume_stats(self):
+        def _emulate_vgs_execute(_command, *_args, **_kwargs):
+            out = "  test1-volumes  5,52  0,52"
+            out += " test2-volumes  5.52  0.52"
+            return out, None
+
+        self.volume.driver.set_execute(_emulate_vgs_execute)
+
+        self.volume.driver._update_volume_status()
+
+        stats = self.volume.driver._stats
+
+        self.assertEquals(stats['total_capacity_gb'], float('5.52'))
+        self.assertEquals(stats['free_capacity_gb'], float('0.52'))
+
+
+class FibreChannelTestCase(DriverTestCase):
+    """Test Case for FibreChannelDriver"""
+    driver_name = "cinder.volume.driver.FibreChannelDriver"
+
+    def test_initialize_connection(self):
+        self.driver = driver.FibreChannelDriver()
+        self.driver.do_setup(None)
+        self.assertRaises(NotImplementedError,
+                          self.driver.initialize_connection, {}, {})
 
 
 class VolumePolicyTestCase(test.TestCase):
