@@ -95,13 +95,7 @@ hp3par_opts = [
 
 class HP3PARCommon():
 
-    stats = {'driver_version': '1.0',
-             'free_capacity_gb': 'unknown',
-             'reserved_percentage': 0,
-             'storage_protocol': None,
-             'total_capacity_gb': 'unknown',
-             'vendor_name': 'Hewlett-Packard',
-             'volume_backend_name': None}
+    stats = {}
 
     # Valid values for volume type extra specs
     # The first value in the list is the default value
@@ -417,26 +411,42 @@ exit
         const = 0.0009765625
 
         if refresh:
-            try:
-                cpg = client.getCPG(self.config.hp3par_cpg)
-                if 'limitMiB' not in cpg['SDGrowth']:
-                    total_capacity = 'infinite'
-                    free_capacity = 'infinite'
-                else:
-                    total_capacity = int(cpg['SDGrowth']['limitMiB'] * const)
-                    free_capacity = int((cpg['SDGrowth']['limitMiB'] -
-                                        cpg['UsrUsage']['usedMiB']) * const)
-
-                self.stats['total_capacity_gb'] = total_capacity
-                self.stats['free_capacity_gb'] = free_capacity
-            except hpexceptions.HTTPNotFound:
-                err = (_("CPG (%s) doesn't exist on array")
-                       % self.config.hp3par_cpg)
-                LOG.error(err)
-                raise exception.InvalidInput(reason=err)
+            self._update_volume_stats(client)
 
         return self.stats
 
+    def _update_volume_stats(self, client):
+
+        # storage_protocol and volume_backend_name are
+        # set in the child classes
+        stats = {'driver_version': '1.0',
+                 'free_capacity_gb': 'unknown',
+                 'reserved_percentage': 0,
+                 'storage_protocol': None,
+                 'total_capacity_gb': 'unknown',
+                 'vendor_name': 'Hewlett-Packard',
+                 'volume_backend_name': None}
+
+        try:
+            cpg = client.getCPG(self.config.hp3par_cpg)
+            if 'limitMiB' not in cpg['SDGrowth']:
+                total_capacity = 'infinite'
+                free_capacity = 'infinite'
+            else:
+                total_capacity = int(cpg['SDGrowth']['limitMiB'] * const)
+                free_capacity = int((cpg['SDGrowth']['limitMiB'] -
+                                    cpg['UsrUsage']['usedMiB']) * const)
+
+            stats['total_capacity_gb'] = total_capacity
+            stats['free_capacity_gb'] = free_capacity
+        except hpexceptions.HTTPNotFound:
+            err = (_("CPG (%s) doesn't exist on array")
+                   % self.config.hp3par_cpg)
+            LOG.error(err)
+            raise exception.InvalidInput(reason=err)
+
+        self.stats = stats
+
     def create_vlun(self, volume, host, client):
         """
         In order to export a volume on a 3PAR box, we have to
